<?php

namespace Mastodon;

require_once 'vendor/autoload.php';

use Appwrite\Client;
use Appwrite\Services\Databases;
use Appwrite\Query;

class Crawler
{
    public static $feedMapper = [
        'profile' => '/api/v1/accounts/$user_id/statuses',
        'bookmarks' => '/api/v1/bookmarks',
        'favourites' => '/api/v1/favourites',
    ];

    public static function crawlSubmissions($user, $client)
    {
        return self::startFeedCrawler('profile', 3, $user, $client);
    }

    public static function startFeedCrawler($feedId, $points, $user, $client)
    {
        $tags = self::crawlFeed($feedId, $user, $client);
        // return $tags;

        if(count($tags) == 0) {
            return [];
        }

        $databases = new Databases($client);
        $query = new Query();

        $hashtagsDB = $databases->listDocuments(getenv("APPWRITE_DATABASE_ID"), 'hashtags', [
            $query->equal('user', $user['$id']),
        ])['documents'];

        $hashtags = [];

        foreach ($hashtagsDB as $tag) {
            $hashtags[$tag['hashtag']] = [
                'id' => $tag['$id'],
                'points' => $tag['points']
            ];
        }

        foreach ($tags as $tag => $details) {
            if (isset($hashtags[$tag])) {
                $databases->updateDocument(getenv("APPWRITE_DATABASE_ID"), 'hashtags', $hashtags[$tag]['id'], [
                    'points' => $hashtags[$tag]['points'] + $details['points'] + $points,
                    'lastSeen' => $details['lastSeen']
                ]);
            } else {
                $data = [
                    'user' => $user['$id'],
                    'hashtag' => $tag,
                    'points' => $details['points'] + $points,
                    'lastseen' => $details['lastSeen']
                ];

                $databases->createDocument(getenv("APPWRITE_DATABASE_ID"), 'hashtags', 'unique()', $data);
            }
        }

        return $tags;
    }

    public static function crawlFeed($feedId, $user, $client)
    {
        $databases = new Databases($client);
        $query = new Query();

        $record = $databases->listDocuments(getenv("APPWRITE_DATABASE_ID"), 'index_history', [
            $query->equal('user_id', $user['$id']),
            $query->equal('feed_name', $feedId),
        ]);

        $request = new \GuzzleHttp\Client(['base_uri' => $user['prefs']['user_server_uri']]);

        $user_account_details = json_decode($user['prefs']['account_details'], true);

        $responseBody = '';

        $uri = self::$feedMapper[$feedId];
        $uri = str_replace('$user_id', $user_account_details['id'], $uri);

        if ($record['total'] == 0) {
            $response = $request->request('GET', $uri, [
                'headers' => [
                    'Authorization' => 'Bearer ' . $user['prefs']['user_token']
                ]
            ]);

            $links = \GuzzleHttp\Psr7\Header::parse($response->getHeader('Link'));

            $responseBody = (string)$response->getBody();
            $responseBody = json_decode($responseBody, true);

            $data = [
                'user_id' => $user['$id'],
                'feed_name' => $feedId,
            ];

            if (count($responseBody) == 0 && !isset($links[0])) {
                $data['last_post_id'] = '0';
                $data['first_post_id'] = '0';

                return [];
            } else if ($links[0]['rel'] == "next") {
                $data['last_post_id'] = rtrim(explode('=', $links[0][0])[1], '>');
                $data['first_post_id'] = rtrim(explode('=', $links[1][0])[1], '>');
                $data['populating_new_index'] = (count($responseBody) < 20) ? false : true;
            } else if ($links[0]['rel'] == "prev") {
                $data['first_post_id'] = rtrim(explode('=', $links[0][0])[1], '>');
                $data['last_post_id'] = '0';
                $data['populating_new_index'] = false;
            } else {
                $data['last_post_id'] = $responseBody[count($responseBody) - 1]['id'];
                $data['first_post_id'] = $responseBody[0]['id'];
                $data['populating_new_index'] = (count($responseBody) < 20) ? false : true;
            }

            $databases->createDocument(getenv("APPWRITE_DATABASE_ID"), 'index_history', 'unique()', $data);
        } else {
            $index = $record['documents'][0];

            if ($index['populating_new_index'] == true) {
                $response = $request->request('GET', "$uri?max_id=" . $index['last_post_id'], [
                    'headers' => [
                        'Authorization' => 'Bearer ' . $user['prefs']['user_token']
                    ]
                ]);

                $links = \GuzzleHttp\Psr7\Header::parse($response->getHeader('Link'));

                $responseBody = (string)$response->getBody();
                $responseBody = json_decode($responseBody, true);

                if(count($responseBody) == 0) {
                    return [];
                }

                $data = [];

                if(isset($links[0])) {
                    if ($links[0]['rel'] == "prev") {
                        $data['first_post_id'] = rtrim(explode('=', $links[0][0])[1], '>');
                        $data['last_post_id'] = '0';
                        $data['populating_new_index'] = false;        
                    } else {
                        $data['last_post_id'] = rtrim(explode('=', $links[0][0])[1], '>');
                        if($index['first_post_id'] == '0') {
                            $data['first_post_id'] = rtrim(explode('=', $links[1][0])[1], '>');
                        }
                        $data['populating_new_index'] = (count($responseBody) < 20) ? false : true;        
                    }
                } else {
                    $data['last_post_id'] = $responseBody[count($responseBody) - 1]['id'];
                    $data['populating_new_index'] = (count($responseBody) < 20) ? false : true;
                }

                $databases->updateDocument(getenv("APPWRITE_DATABASE_ID"), 'index_history', $index['$id'], $data);
            } else {
                $response = $request->request('GET', "$uri?min_id=" . $index['first_post_id'], [
                    'headers' => [
                        'Authorization' => 'Bearer ' . $user['prefs']['user_token']
                    ]
                ]);

                $links = \GuzzleHttp\Psr7\Header::parse($response->getHeader('Link'));

                $responseBody = (string)$response->getBody();
                $responseBody = json_decode($responseBody, true);

                if(count($responseBody) == 0) {
                    return [];
                }

                $data = [];

                if(isset($links[0])) {
                    if ($links[0]['rel'] == "prev") {
                        $data['first_post_id'] = rtrim(explode('=', $links[0][0])[1], '>');
                    } else {
                        // $data['last_post_id'] = rtrim(explode('=', $links[0][0])[1], '>');
                        $data['first_post_id'] = rtrim(explode('=', $links[1][0])[1], '>');
                    }
                } else {
                    $data['first_post_id'] = $responseBody[0]['id'];
                }

                $databases->updateDocument(getenv("APPWRITE_DATABASE_ID"), 'index_history', $index['$id'], $data);
            }
        }

        $tags = [];

        foreach ($responseBody as $post) {
            if (isset($post['reblog']['id'])) {
                foreach ($post['reblog']['tags'] as $tag) {
                    if (!isset($tags[$tag['name']])) {
                        $tags[$tag['name']]['points'] = 0;
                    }
                    $tags[$tag['name']]['lastSeen'] = $post['created_at'];
                    $tags[$tag['name']]['points']++;
                }
            } else {
                foreach ($post['tags'] as $tag) {
                    if (!isset($tags[$tag['name']])) {
                        $tags[$tag['name']]['points'] = 0;
                    }
                    $tags[$tag['name']]['lastSeen'] = $post['created_at'];
                    $tags[$tag['name']]['points']++;
                    $tags[$tag['name']]['points']++;
                }
            }
        }

        return $tags;
    }
}